/*
 * Created on 26-Jan-2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package com.dammfine.xmltidy;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;

/**
 * @author Ben Damm
 * 
 */
public class Main
{
	private static Logger log = Logger.getLogger(Main.class);

	public static void main(String[] args)
	{
		// Load log4j
		new PropertyConfigurator();
		
		//BasicConfigurator.configure();
		log.debug("App starting.");
		
		try
		{
			main2(args);
		}
		catch( Exception e )
		{
			log.error("Error in main.", e);
		}
	}
	
	public static void usage()
	{
		ulog("xmltidy - Simple xml cleaner upper");
		ulog("");
		ulog("java -jar xmltidy.jar --input <input.xml> [options]");
		ulog("");
		ulog(" Options: ");
		ulog("--nooutput", "Go through all motions but produce nothing.");
		ulog("--output <output.xml>", "Specify output file (default is stdout)." );
		ulog("--nochange", "Perform analysis only, make no changes." );
		ulog("", "--nochange assumes --nooutput." );
	}	
	
	/* Flags that require a parameter have String values.
	 * Flags that don't have Boolean values.  Flags 
	 * come in two flavours; --value, and --novalue.  The parsing
	 * code will set the Boolean as appropriate on non-parameter
	 * arguments, but will set null on arguments that require
	 * a parameter.
	 * 
	 * ie,
	 * "--change" sets TRUE for the "--change" arg.
	 * "--nochange" sets FALSE for the "--change" arg.
	 * "--output <value>" sets the value for the --output param.
	 * "--output" followed by nothing is an error.
	 * "--nooutput" sets null for the --output param.
	 * 
	 * */
	public static final String ARG_INPUT = "--input";
	public static final String ARG_OUTPUT = "--output";
	public static final String ARG_CHANGE = "--change";
	
	private static Map makeArgsMap()
	{
		Map m = new HashMap();
		
		addArg( m, ARG_INPUT, true, "-" );
		addArg( m, ARG_OUTPUT, true, "-" );
		addArg( m, ARG_CHANGE, false, Boolean.TRUE );
		
		// Main option parsing code is towards bottom.
		return m;
	}
	
	
	public static void main2(String args[]) throws Exception
	{
		Map p = getProperties(args);
		instance = new Main( p );
	
		instance.run();
	}
	
	public void run()
		throws Exception
	{ 
		boolean makeChange = getArgBoolean(ARG_CHANGE);
		
		if( ! makeChange )
		{
			log.info("No changes and no output.");
		}
		
		// no validate, because of the network traffic.
		SAXBuilder builder = new SAXBuilder(false);
		builder.setValidation(false);
		//DTDHandler dtdHandler = new NoOpDTDHandler();
		//builder.setDTDHandler(dtdHandler);
		
		// This is to tell the XML parser to NOT perform a network
		// hit on the xbel server to get the DTD.
		// Side effect, all entities vanish.
		//builder.setEntityResolver(new NoOpEntityResolver());
		
		// The following line only works with xerces, but
		// it is a better way to do it.
		builder.setFeature( "http://apache.org/xml/features/nonvalidating/load-external-dtd", false);
		
		String filename = getArgString(ARG_INPUT);
		InputStream inputStream = null;
		if( filename.equals("-") )
		{
			inputStream = System.in;
		}
		else
		{
			inputStream = new FileInputStream( filename );
		}
		
		Document doc = builder.build(inputStream);
		
		Element root = doc.getRootElement();
		
		log.debug( "Done loading." );
		
		/**
		 * Do stuff
		 * */
	
		XmlFormat xForm = new XmlFormat();
		String out = getArgString(ARG_OUTPUT);
		if( out == null || !makeChange )
		{
			// We're done
			return;
		}
		
		OutputStream outS = null;
		if( out.equals("-") )
		{
			outS = System.out;
		}
		else
		{
			outS = new FileOutputStream(out);
		}
		
		xForm.output(doc, outS);
	}
	
	public static Main instance;
	
	public Main( Map p )
	{
		this.props = p;
	}
	
	private Map props;
	
	public static Main getInstance()
	{
		return instance;
	}
	
	public String getArgString( String argKey )
	{
		return (String)props.get( argKey );
	}
	
	public boolean getArgBoolean( String argKey )
	{
		return ((Boolean)props.get( argKey )).booleanValue();
	}
	
	public static Map getProperties( String argsarr[] )
	{
		Map p = new HashMap();
		Map options = makeArgsMap();
		
		LinkedList args = new LinkedList( Arrays.asList( argsarr ) );
		while( ! args.isEmpty() )
		{
			String argStr = (String)args.removeFirst();
			boolean noVersion = false;
			
			if( argStr.startsWith("--no") )
			{
				argStr = "--" + argStr.substring(4);
				noVersion = true;
			}
			
			// Check for unknown flags, and exit if we find one.
			Arg arg = (Arg)options.get( argStr );
			if( arg == null )
			{
				usage();
				System.exit(1);
			}
			
			if( arg.getTakesArg() )
			{
				if( noVersion )
				{
					p.put( argStr, null );
				}
				else
				{
					String next = (String)args.removeFirst();
					p.put( argStr, next );
				}
			}
			else
			{
				if( noVersion )
				{
					p.put( argStr, Boolean.FALSE );
				}
				else
				{
					p.put( argStr, Boolean.TRUE );
				}
			}
			
			options.remove( argStr );
		}
		
		// Now we are done, fill in the default values that remain.
		Iterator i = options.keySet().iterator();
		while( i.hasNext() )
		{
			String key = (String)i.next();
			Arg value = (Arg)options.get(key);
			
			p.put( key, value.getDefault() );
		}
		
		
		return p;
	}
	

	private static void addArg( Map m, String longForm, boolean takeArg, Object def )
	{
		m.put(longForm, new Arg( longForm, takeArg, def ));
	}
	
	private static class Arg
	{
		private String longForm;
		private boolean takeArg;
		private Object value;
		
		public Arg( String form, boolean takeArg )
		{
			this( form, takeArg, null );
		}
		
		
		public Arg( String form, boolean takeArg, Object def )
		{
			this.longForm = form;
			this.takeArg = takeArg;
			this.value = def;
			
			if( def == null && !takeArg )
			{
				this.value = Boolean.FALSE;
			}
		}
		
		public boolean getTakesArg()
		{
			return takeArg;
		}
		
		public Object getDefault()
		{
			return value;
		}
	}
	
	
	private static void ulog( String s )
	{
		System.out.println(s);
	}
	
	private static void ulog( String col1, String col2 )
	{
		System.out.print("  " + col1);
		int spaces = col1.length();
		System.out.print("                                ".substring(spaces) );
		System.out.println(col2);
	}
	
	
}

